#include "display.h"
#include "timer.h"
#include "messages.h"
#include "detector.h"
#include "monitor.h"
#include "config.h"
#include "util.h"

#define DISPLAY_DIGIT_COUNT  3

static void UpdateDisplay(struct MultiplexDisplay *display, uint8_t pulse)
{
	SetDisplayValue(display, pulse);

	uint8_t threshold = GetBlinkThreshold();
	if (threshold > 0)
	{
		SetDisplayMode(display, (pulse >= threshold) ? DISPLAY_BLINKING : DISPLAY_STATIC_ON);
	}
	else
	{
		SetDisplayMode(display, DISPLAY_STATIC_ON);
	}
}

int main(void)
{
	InitGpioItems();
	InitMessageQueue();

	struct PulseDetector detector;
	InitPulseDetector(&detector, GPIO_ITEM_LOG_PULSE);

	// Digits going from right to left, digits[0] = 10^0, digits[1] = 10^1, etc.
	// because of mirrored connector layout (top vs. bottom layer)
	struct DisplayDigit digits[DISPLAY_DIGIT_COUNT];
	InitDigit(&digits[0], GPIO_ITEM_DRIVER_3);
	InitDigit(&digits[1], GPIO_ITEM_DRIVER_2);
	InitDigit(&digits[2], GPIO_ITEM_DRIVER_1);
	struct MultiplexDisplay display;
	InitDisplay(&display, GPIO_ITEM_SEGMENTS, digits, ARRAY_SIZE(digits));

	struct PulseMonitor monitor;
	InitPulseMonitor(&monitor);

	InitTimer();
	EnableInterrupts();

	for (;;)
	{
		enum MessageId msg;
		if (GetMessage(&msg))
		{
			switch (msg)
			{
			case MSG_DISPLAY_MULTIPLEX_TICK:
				DisplayMultiplexTick(&display);
				break;
			case MSG_DISPLAY_BLINKING_TICK:
				DisplayBlinkingTick(&display);
				break;
			case MSG_LOG_PULSE_SAMPLE_TICK:
				SamplePulse(&detector);
				SetGpioItemValue(GPIO_ITEM_PULSE_STATE, GetSampleState(&detector));
				break;
			case MSG_PULSE_DETECTED:
				HandlePulseMonitorMessage(&monitor, msg);
				break;
			case MSG_DISPLAY_UPDATE_TICK:
				HandlePulseMonitorMessage(&monitor, msg);
				UpdateDisplay(&display, GetMonitorValue(&monitor));
				break;
			}
		}
	}
}
