#include <stdint.h>

#include "messages.h"
#include "util.h"

#define MESSAGE_QUEUE_SIZE 5

struct MessageQueue
{
	enum MessageId messages[MESSAGE_QUEUE_SIZE];
	uint8_t readPos;
	uint8_t writePos;
	bool empty;
	bool full;
};

static struct MessageQueue TheMessageQueue;

void InitMessageQueue(void)
{
	TheMessageQueue.readPos = 0;
	TheMessageQueue.writePos = 0;
	TheMessageQueue.empty = true;
	TheMessageQueue.full = false;
}

void PostMessage(enum MessageId msg)
{
	uint8_t sreg = DisableInterrupts();

	if (!TheMessageQueue.full)
	{
		TheMessageQueue.messages[TheMessageQueue.writePos] = msg;
		if (++TheMessageQueue.writePos >= MESSAGE_QUEUE_SIZE)
		{
			TheMessageQueue.writePos = 0;
		}
		TheMessageQueue.empty = false;
		TheMessageQueue.full = TheMessageQueue.writePos == TheMessageQueue.readPos;
	}

	RestoreInterrupts(sreg);
}

bool GetMessage(enum MessageId *msg)
{
	bool result = false;

	uint8_t sreg = DisableInterrupts();

	if (!TheMessageQueue.empty)
	{
		*msg = TheMessageQueue.messages[TheMessageQueue.readPos];
		if (++TheMessageQueue.readPos >= MESSAGE_QUEUE_SIZE)
		{
			TheMessageQueue.readPos = 0;
		}
		TheMessageQueue.empty = TheMessageQueue.writePos == TheMessageQueue.readPos;
		TheMessageQueue.full = false;
		result = true;
	}

	RestoreInterrupts(sreg);

	return result;
}
