#include "util.h"
#include "monitor.h"

static void IncrementCurrentCounter(struct PulseMonitor *monitor)
{
	if (monitor->index < ARRAY_SIZE(monitor->counters))
	{
		monitor->counters[monitor->index]++;
	}
}

static uint8_t GetCurrentCounter(struct PulseMonitor *monitor)
{
	uint8_t count = 0;

	if (monitor->index < ARRAY_SIZE(monitor->counters))
	{
		count = monitor->counters[monitor->index];
	}

	return count;
}

static void SwitchCurrentCounter(struct PulseMonitor *monitor)
{
	monitor->index++;
	if (monitor->index >= ARRAY_SIZE(monitor->counters))
	{
		monitor->index = 0;
	}
	monitor->counters[monitor->index] = 0;
}

static void ResetCounters(struct PulseMonitor *monitor)
{
	monitor->index = 0;
	for (uint8_t i = 0; i < ARRAY_SIZE(monitor->counters); i++)
	{
		monitor->counters[i] = 0;
	}
}

void InitPulseMonitor(struct PulseMonitor *monitor)
{
	ResetCounters(monitor);
	monitor->state = MONITOR_IDLE;
}

void HandlePulseMonitorMessage(struct PulseMonitor *monitor, enum MessageId msg)
{
	switch (monitor->state)
	{
	case MONITOR_IDLE:
		switch (msg)
		{
		case MSG_PULSE_DETECTED:
			monitor->state = MONITOR_LISTENING;
			break;
		case MSG_DISPLAY_UPDATE_TICK:
			/* stay in IDLE */
			break;
		default:
			break;
		}
		break;
	case MONITOR_LISTENING:
		switch (msg)
		{
		case MSG_PULSE_DETECTED:
			/* stay in LISTENING */
			break;
		case MSG_DISPLAY_UPDATE_TICK:
			monitor->state = MONITOR_ACTIVE;
			break;
		default:
			break;
		}
		break;
	case MONITOR_ACTIVE:
		switch (msg)
		{
		case MSG_PULSE_DETECTED:
			IncrementCurrentCounter(monitor);
			break;
		case MSG_DISPLAY_UPDATE_TICK:
			if (GetCurrentCounter(monitor) > 0)
			{
				SwitchCurrentCounter(monitor);
			}
			else
			{
				ResetCounters(monitor);
				monitor->state = MONITOR_IDLE;
			}
			break;
		default:
			break;
		}
		break;
	}
}

uint8_t GetMonitorValue(struct PulseMonitor *monitor)
{
	uint8_t result = 0;

	if (monitor->state == MONITOR_ACTIVE)
	{
		uint16_t sum = 0;
		uint8_t count = 0;

		for (uint8_t i = 0; i < ARRAY_SIZE(monitor->counters); i++)
		{
			if (monitor->counters[i])
			{
				sum += monitor->counters[i];
				count++;
			}
		}

		if (count > 0)
		{
			result = (sum * ARRAY_SIZE(monitor->counters)) / count;
		}
	}

	return result;
}
