#include <avr/io.h>
#include <avr/interrupt.h>
#include "timer.h"
#include "util.h"
#include "config.h"

#include "messages.h"

struct TimerEntry
{
	const enum MessageId msgId;
	const uint16_t period;
	uint16_t counter;
};

static struct TimerEntry Timers[] =
{
	/* msgId                        period                           counter */
	{  MSG_DISPLAY_MULTIPLEX_TICK,  CFG_DISPLAY_MULTIPLEX_PERIOD,    0       },
	{  MSG_DISPLAY_BLINKING_TICK,   CFG_DISPLAY_BLINKING_PERIOD,     0       },
	{  MSG_LOG_PULSE_SAMPLE_TICK,   CFG_PULSE_SAMPLE_PERIOD,         0       },
	{  MSG_DISPLAY_UPDATE_TICK,     60000 / CFG_UPDATES_PER_MINUTE,  0       },
};

ISR(TIMER0_COMPA_vect)
{
	for (uint8_t i = 0; i < ARRAY_SIZE(Timers); i++)
	{
		struct TimerEntry *timer = &Timers[i];
		
		if (timer->counter > 0)
		{
			timer->counter--;
		}
		if (timer->counter == 0)
		{
			timer->counter = timer->period;
			PostMessage(timer->msgId);
		}
	}
}

void InitTimer(void)
{
	// CTC mode
	TCCR0A = (1u << WGM01);
	// Prescaler 1/64
	TCCR0B = (1u << CS01) | (1u << CS00);
	// 8000000 Hz / 64 / 125 = 1000 Hz => 1ms
	OCR0A = 125;
	// Enable COMPA interrupts
	TIMSK |= (1u << OCIE0A);
}
